// cc -I/usr/include/drm -ldrm drift_monitor.c -o drift_monitor
//
// drift_monitor: monitors drift of VBLANK time against regular time
//

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <xf86drm.h>
#include <radeon_drm.h>

#define A(a) (*(argv + (a)) ? **(argv + (a)) : 0)

main(argc, argv)
    char **argv;
{
    int fd;
    struct timeval tval;
    drm_radeon_vsync_t vsync;
    drm_radeon_setparam_t vbl_activate;
    int drift_speed, last_drift_speed = 0, drift, last_drift = 0;

    if ((fd = drmOpen("radeon", 0)) < 0) {
	printf("drmOpen: %s\n", strerror(errno)); exit(-1);
    }
#if 0 /* not here */
    vbl_activate.param = RADEON_SETPARAM_VBLANK_CRTC;
    vbl_activate.value = DRM_RADEON_VBLANK_CRTC1;
    if (ioctl(fd, DRM_IOCTL_RADEON_SETPARAM, &vbl_activate)) {
        printf("DRM_IOCTL_RADEON_SETPARAM: %s\n", strerror(errno)); exit(-1);
    }
#endif
    while (1) {
	vsync.vbl_trim = VBL_IGNORE;
	if (ioctl(fd, DRM_IOCTL_RADEON_VSYNC, &vsync)) {
	    printf("DRM_IOCTL_RADEON_VSYNC: %s\n", strerror(errno)); exit(-1);
	}
	tval.tv_sec = vsync.vbl_received * 20 / 1000;
	tval.tv_usec = vsync.vbl_received * 20 % 1000 * 1000 + vsync.vbl_since.tv_usec;
	if (A(1) == 'd') {
	    printf("vbl_now:         %10d.%06d\n", vsync.vbl_now.tv_sec, vsync.vbl_now.tv_usec);
	    printf("vbl_since:       %10d.%06d\n", vsync.vbl_since.tv_sec, vsync.vbl_since.tv_usec);
	    printf("vbl_received:    %10d\n", vsync.vbl_received);
	    printf("vbl_trim:        0x%08x\n", vsync.vbl_trim);
	    printf("since vbl start: %10d.%06d\n", tval.tv_sec, tval.tv_usec);
	}
	drift = vsync.vbl_now.tv_usec - tval.tv_usec;
	if (drift < 0) drift += 1000000;
	if (last_drift) {
	    drift_speed = drift - last_drift;
	    printf("%s%d", drift_speed >= 0 ? "+" : "", drift_speed);
	    printf("%s\n", abs(drift_speed) > 1000 ? ", excessive drift speed" : "");
	    if (last_drift_speed) {
		if (abs(drift_speed - last_drift_speed) > 1000) {
		    printf("drift_speed too unstable\n");
		}
	    }
	    last_drift_speed = drift_speed;
	} else {
	    printf("not available\n");
	}
	last_drift = drift;
	sleep(1);
    }
}

